/*
 *  ALTerator - ALT Linux configuration project
 *
 *  Copyright (c) 2004,2005 ALT Linux Ltd.
 *  Copyright (c) 2004,2005 Alexey Voinov
 *  Copyright (c) 2004,2005 Stanislav Ievlev
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *  USA.
 */
#include <dialog.hh>
#include <command_set.hh>
#include <xmlscope.hh>

#include <iostream>
#include <fstream>

namespace
{
    struct parse_attr
    {
	parse_attr(command& cmd): cmd_(cmd) {}

	void operator()(const std::string& name, const std::string& value)
	{
		if(name == "name") return;
		cmd_(name, value);
	}

    private:
	command& cmd_;
    };

    struct parse_param
    {
	parse_param(command& cmd): cmd_(cmd) {}

	void operator()(xml::node& n) const
	{
		if(n.type != xml::node::normal || n.name != "param") return;
		if(!n.prop_exists("name")) return;
		std::string name = n.prop_get("name");

		std::string value;
		if(n.prop_exists("value"))
			value = n.prop_get("value");
		else
			value = n.value_get();

		cmd_(name, value);
	}

    private:
	command& cmd_;
    };

    struct parse_command
    {
	parse_command(command_set& c): c_(c), ndx_(-1) {}

	void operator()(xml::node& n)
	{
		if(n.type != xml::node::normal || n.name != "command") return;
		if(!n.prop_exists("name")) return;

		command cmd(n.prop_get("name"));

		std::for_each(n.attrs.begin(), n.attrs.end(),
					unpair<parse_attr>(parse_attr(cmd)));

		std::for_each(n.nodes.begin(), n.nodes.end(),
					parse_param(cmd));

		if(ndx_ < 0) ndx_ = c_.add(cmd);
		else c_.add(ndx_, cmd);
	}

	operator long() const { return ndx_; }

    private:
	command_set& c_;
	long ndx_;
    };

    struct generate_id
    {
	generate_id(void): id_('a') {}
	std::string operator()(const std::string& prefix)
	{
	    if (prefix != "hbox/" && prefix!= "vbox/")
		return prefix + id_++;
	    else
		return prefix;
	}

    private:
	char id_;
    };

    inline
    std::string attr_or_value(xml::node& n,const std::string& name)
    {
	return n.prop_exists(name)?n.prop_get(name):n.value_get();
    }

    struct apply_attrs
    {
	apply_attrs(const std::string& id,dialog& d,evaluator& e):
	    id_(id),
	    d_(d),
	    e_(e)
	{}
	
	void operator()(const std::string& name, const std::string& value)
	{
	    d_.set_attr(id_,name,e_(value));
	}
	
	const std::string& id_;
	dialog& d_;
	evaluator& e_;
    };

    struct parse_dialog
    {
	parse_dialog(dialog& d, command_set& c,generate_id& ids,evaluator& e):
	    d_(d),
	    c_(c),
	    ids_(ids),
	    e_(e)
	{}

	void operator()(xml::node& n)
	{
		if (n.type != xml::node::normal) return;
	
		std::string id = n.prop_exists("id")?n.prop_get("id"):ids_(n.name+"/");
		long cmdidx = std::for_each(n.nodes.begin(), n.nodes.end(),
					    parse_command(c_));
		
		const std::string evalue=e_(n.value_get());
		
		if("label" == n.name)
			d_.add_label(id,evalue);
		else if("variable" == n.name)
			d_.add_variable(id,evalue);
		else if("button" == n.name)
			d_.add_button(id,evalue,cmdidx);
		else if("listbox" == n.name)
			d_.add_listbox(id,evalue,cmdidx);
		else if("textbox" == n.name)
			d_.add_textbox(id,evalue,cmdidx);
		else if("hbox" == n.name)
		{
			d_.begin_hbox();
			std::for_each(n.nodes.begin(),n.nodes.end(), parse_dialog(d_, c_,ids_,e_));
			d_.end_hbox();
		}
		else if("vbox" == n.name)
		{
			d_.begin_vbox();
			std::for_each(n.nodes.begin(),n.nodes.end(), parse_dialog(d_, c_,ids_,e_));
			d_.end_vbox();
		}
		
		std::for_each(n.attrs.begin(), n.attrs.end(), unpair(apply_attrs(id,d_,e_)));
	}

    private:
	dialog& d_;
	command_set& c_;
	generate_id &ids_;
	evaluator   &e_;
    };
} //namespace

command_set load_dialog(const std::string& filename, dialog& d,evaluator& e)
{
	command_set c;

	std::ifstream is(filename.c_str());
	if(!is) return c;

	xml::nodes_t data;
	xml::load(is, data);
	xml::process_ns(data);
	xml::process_xi(data, "./", "/");
	generate_id ids_generator;

	std::for_each(data.begin(), data.end(), parse_dialog(d, c, ids_generator,e));
	return c;
}
